@@ -4,17 +4,24 @@
  * given, provided that this copyright is kept intact.
  */
 
-#include "FvwmScript.h"
+#include "types.h"
+
+#ifdef DEBUG			/* For debugging */
+#include <unchecked.h>
+#endif
 
 /* Variables utilise par l'analyseur syntaxique */
 extern ScriptProp *scriptprop;
 extern int nbobj;			/* Nombre d'objets */
 extern int numligne;			/* Numero de ligne */
-extern TabObj *tabobj;			/* Tableau d'objets, limite=30 */
+extern TabObj *tabobj;			/* Tableau d'objets, limite=100 */
 extern char **TabVVar;			/* Tableau des variables du sript */
-extern int TabIdObj[31];
+extern int TabIdObj[101];
 extern Bloc **TabIObj;
 extern CaseObj *TabCObj;
+#ifdef DEBUG
+extern int __bounds_debug_no_checking;
+#endif
 
 /* Constante de couleurs utilise dans le tableau TabColor */
 #define black 0
@@ -27,20 +34,20 @@ extern CaseObj *TabCObj;
 
 /* Variables globales */
 char *ScriptName;		/* Nom du fichier contenat le script decrivant le GUI */
+char *ScriptPath;
+char *ModuleName;
 int fd[2]; 			/* pipe pair */
 int fd_err;
 int x_fd;			/* fd for X */
 Window ref;
 
-/*void (*TabCom[30]) (int NbArg,long *TabArg);
-char *(*TabFunc[10]) (int *NbArg, long *TabArg);
-int (*TabComp[15]) (char *arg1,char *arg2);*/
 
 extern void (*TabCom[30]) (int NbArg,long *TabArg);
 
 X11base *x11base;		/* Pour le serveur X */
+TypeBuffSend BuffSend;		/* Pour les communication entre script */
 int grab_server = 0;
-struct XObj *tabxobj[30];
+struct XObj *tabxobj[100];
 char *Scrapt;
 Atom propriete,type;
 static Atom wm_del_win;
@@ -58,7 +65,6 @@ void Debug()
    /* Execution du bloc d'instruction */
    fprintf(stderr,"Id de l'objet %d\n",TabIdObj[j]);
    fprintf(stderr,"Nb Instruction %d\n",TabIObj[TabIdObj[j]][i].NbInstr);
-   fprintf(stderr,"Adresse du bloc %d\n",TabIObj[TabIdObj[j]][i].TabInstr);
   }
 
 }
@@ -66,13 +72,14 @@ void Debug()
 /* Lecture du fichier contenant le scipt */
 void ReadConfig (char *ScriptName)
 {
-  extern FILE *yyin,*yyout;
-  int i;
+  extern FILE *yyin;
+  char s[255];
 
-  yyin=fopen(ScriptName,"r");
+  sprintf(s,"%s/%s",ScriptPath,ScriptName);
+  yyin=fopen(s,"r");
   if (yyin == NULL) 
   {
-   fprintf(stderr,"Can't open the script %s",ScriptName);
+   fprintf(stderr,"Can't open the script %s",s);
    exit(1);
   }
   /* On ne redefini pas yyout qui est la sortie standard */
@@ -84,24 +91,16 @@ void ReadConfig (char *ScriptName)
   close((int)yyin);
 }
 
+/* Quitter par l'option Delete du bouton de la fenetre */
 void DeadPipe(int nonsense)
 {
-  int i;
-
-  signal(SIGPIPE, SIG_IGN);/* Xsync may cause SIGPIPE */
-  /* delete swallowed windows, but not modules (fvwm handles those) */
-  /* if((Buttons[button].swallow == 3)&&(Buttons[button].module == 0)) */
-  /* the above doesn't seem to hold true any more - CKH */
-  for (i=0;i<nbobj;i++)
-   tabxobj[i]->DestroyObj(tabxobj[i]);
-  XSync(x11base->display,0);
-  exit(0);
+ Quit (0,NULL);
 }
 
 /* Lecture du fichier system.fvwmrc ou .fvwmrc */
 void ParseOptions(void)
 {
-  char *tline,*tmp;
+  char *tline;
 
   GetConfigLine(fd,&tline);
   while(tline != (char *)0)
@@ -110,16 +109,28 @@ void ParseOptions(void)
       {
 	CopyString(&pixmapPath,&tline[10]);
       }
+      
+      if((strlen(&tline[0])>1)&&(mystrncasecmp(tline,"*FvwmScriptPath",15)==0))
+      {
+	CopyString(&ScriptPath,&tline[15]);
+      }
       GetConfigLine(fd,&tline);
     }
 }
 
-/* Procedure d'initialisation du serveur X et des variables globales*/
-void Xinit()
-{
 
+/* Procedure d'initialisation du serveur X et des variables globales*/
+void Xinit(int IsFather)
+{
+ char *name;
+ Atom myatom;
+ int i=16;
 
  /* Connextion au serveur X */
+#ifdef DEBUG
+ __bounds_debug_no_checking=True;
+#endif
+
  x11base->display=XOpenDisplay(NULL);
  if (x11base->display==NULL) 
  {
@@ -127,7 +138,25 @@ void Xinit()
   exit(1);
  }
 
- 
+#ifdef DEBUG
+ __bounds_debug_no_checking=False;
+#endif
+
+ if (IsFather)
+ {
+  name=(char*)calloc(sizeof(char),strlen("FvwmScript")+5);
+  do
+  {
+   sprintf(name,"%c%xFvwmScript",161,i);
+   i++;
+   myatom=XInternAtom(x11base->display,name,False);
+  }
+  while (XGetSelectionOwner(x11base->display,myatom)!=None);
+  x11base->TabScriptId[1]=name;
+  x11base->TabScriptId[0]=NULL;
+ }
+
+ x11base->NbChild=0;
  x11base->screen=DefaultScreen(x11base->display);
  x11base->WhitePix=WhitePixel(x11base->display,x11base->screen);
  x11base->BlackPix=BlackPixel(x11base->display,x11base->screen);
@@ -172,38 +201,43 @@ void LoadIcon(struct XObj *xobj)
  }
 }
 
+int MyAllocNamedColor(Display *display,Colormap colormap,char* colorname,XColor* color)
+{
+ static XColor TempColor;
+
+ if (colorname[0]=='#')
+ {
+  if (XParseColor(display,colormap,colorname,color))
+   return XAllocColor(display,colormap,color);
+  else
+   return 0;
+ }
+ else
+ {
+  if (XLookupColor(display,colormap,colorname,&TempColor,color))
+   return XAllocColor(display,colormap,color);
+  else
+   return 0;
+ }
+ return 0;
+}
+
 /* Ouvre une fenetre pour l'affichage du GUI */
 void OpenWindow ()
 {
  XTextProperty Name;
  XWMHints *IndicWM;
  XSizeHints *IndicNorm;
- int FaitColor={DoRed|DoGreen|DoBlue};
- XColor TempColor;
  unsigned long mask;
  XSetWindowAttributes Attr;
 
  /* Allocation des couleurs */
- if (!XAllocNamedColor(x11base->display,x11base->colormap,
-	x11base->forecolor,&x11base->TabColor[fore],&TempColor)) 
-   fprintf(stderr,"Can't alloc named color.\n");
- if (!XAllocNamedColor(x11base->display,x11base->colormap,
-	x11base->backcolor,&x11base->TabColor[back],&TempColor))
-   fprintf(stderr,"Can't alloc named color.\n");
-
- if (!XAllocNamedColor(x11base->display,x11base->colormap,
-	x11base->shadcolor,&x11base->TabColor[shad],&TempColor)) 
-   fprintf(stderr,"Can't alloc named color.\n");
- if (!XAllocNamedColor(x11base->display,x11base->colormap,
-	x11base->licolor,&x11base->TabColor[li],&TempColor))
-   fprintf(stderr,"Can't alloc named color.\n");
-
- if (!XAllocNamedColor(x11base->display,x11base->colormap,
-	"black",&x11base->TabColor[black],&TempColor)) 
-   fprintf(stderr,"Can't alloc named color.\n");
- if (!XAllocNamedColor(x11base->display,x11base->colormap,
-	"white",&x11base->TabColor[white],&TempColor))
-   fprintf(stderr,"Can't alloc named color.\n");
+ MyAllocNamedColor(x11base->display,x11base->colormap,x11base->forecolor,&x11base->TabColor[fore]);
+ MyAllocNamedColor(x11base->display,x11base->colormap,x11base->backcolor,&x11base->TabColor[back]);
+ MyAllocNamedColor(x11base->display,x11base->colormap,x11base->shadcolor,&x11base->TabColor[shad]);
+ MyAllocNamedColor(x11base->display,x11base->colormap,x11base->licolor,&x11base->TabColor[li]);
+ MyAllocNamedColor(x11base->display,x11base->colormap,"#000000",&x11base->TabColor[black]);
+ MyAllocNamedColor(x11base->display,x11base->colormap,"#FFFFFF",&x11base->TabColor[white]);
 
  /* Definition des caracteristiques de la fentre */
  mask=0;
@@ -224,15 +258,6 @@ void OpenWindow ()
  XSetWindowColormap(x11base->display,x11base->win,x11base->colormap);
  x11base->gc=XCreateGC(x11base->display,x11base->win,0,NULL);
 
- /* Chargement des polices */
- if ((x11base->xfont=XLoadQueryFont(x11base->display,x11base->font))==NULL)
-  {
-   fprintf(stderr,"Can't load font %s\n",x11base->font);
-  }
- else
-  XSetFont(x11base->display,x11base->gc,x11base->xfont->fid);
-
-
  /* Choix des evts recus par la fenetre */
  XSelectInput(x11base->display,x11base->win,KeyPressMask|ButtonPressMask|
 	ExposureMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask|ButtonMotionMask);
@@ -262,7 +287,7 @@ void OpenWindow ()
  IndicWM->flags=InputHint|StateHint;
  XSetWMProperties(x11base->display,x11base->win,&Name,
        &Name,NULL,0,IndicNorm,IndicWM,NULL);
- Scrapt=(char*)calloc(1,1000);
+ Scrapt=(char*)calloc(sizeof(char),1);
  
  /* Construction des atomes pour la communication inter-application */
  propriete=XInternAtom(x11base->display,"Prop_selection",False);
@@ -286,12 +311,10 @@ void ExecBloc(Bloc *bloc)
 
 
 /* Construction de l'interface graphique */
-void BuildGUI()
+void BuildGUI(int IsFather)
 {
  int i;
 
- x11base=(X11base*) calloc(1,sizeof(X11base));
-
 
  if (scriptprop->font==NULL)
   x11base->font=strdup("fixed");
@@ -328,7 +351,7 @@ void BuildGUI()
 
  /* Initialisation du serveur X et de la fenetre */
 
- Xinit();
+ Xinit(IsFather);
  OpenWindow();
 
 
@@ -342,10 +365,14 @@ void BuildGUI()
   tabxobj[i]->y=(*tabobj)[i].y;
   tabxobj[i]->width=(*tabobj)[i].width;
   tabxobj[i]->height=(*tabobj)[i].height;
+  if (tabxobj[i]->width==0) tabxobj[i]->width=1;
+  if (tabxobj[i]->height==0) tabxobj[i]->height=1;
   tabxobj[i]->value=(*tabobj)[i].value;
   tabxobj[i]->value2=(*tabobj)[i].value2;
   tabxobj[i]->value3=(*tabobj)[i].value3;
-  tabxobj[i]->state=(*tabobj)[i].state;
+  tabxobj[i]->flags[0]=(*tabobj)[i].flags[0];
+  tabxobj[i]->flags[1]=(*tabobj)[i].flags[1];
+  tabxobj[i]->flags[2]=(*tabobj)[i].flags[2];
   tabxobj[i]->icon=(*tabobj)[i].icon;
   tabxobj[i]->swallow=(*tabobj)[i].swallow;
 
@@ -389,9 +416,13 @@ void BuildGUI()
   tabxobj[i]->icon_maskPixmap=None;
 
   LoadIcon(tabxobj[i]);				/* Chargement de l'icone du widget */
+
   tabxobj[i]->InitObj(tabxobj[i]);
  }
 
+ /* Enregistrement du bloc de taches periodic */
+ x11base->periodictasks=scriptprop->periodictasks;
+
  /*Si un bloc d'initialisation du script existe, on l'execute ici */
  if (scriptprop->initbloc!=NULL)
  {
@@ -404,8 +435,8 @@ void BuildGUI()
  free(scriptprop);
  XMapRaised(x11base->display,x11base->win);
  for (i=0;i<nbobj;i++)
-  if (tabxobj[i]->state!=Hiden)
-   XMapWindow(x11base->display,tabxobj[i]->ObjWin);
+  if (tabxobj[i]->flags[0]!=True)
+   XMapWindow(x11base->display,tabxobj[i]->win);
 
 }
 
@@ -425,14 +456,61 @@ void SendMsg(struct XObj *xobj,int TypeMsg)
   }
 }
 
+/*******************************************/
+/* Appeler lors d'une demande de selection */
+/*******************************************/
+void SendMsgToScript(XEvent event)
+{
+ Atom Sender,Receiver=None;
+ static XEvent evnt_sel;
+ int i;
+
+ Sender=XInternAtom(x11base->display,x11base->TabScriptId[1],True);
+
+ if (event.xselectionrequest.selection==Sender)
+ {
+  i=0;
+  while ((i<BuffSend.NbMsg)&&(event.xselectionrequest.target!=Receiver))
+  {
+   Receiver=XInternAtom(x11base->display,BuffSend.TabMsg[i].R,True);
+   i++;
+  }
+  i--;
+
+  evnt_sel.type=SelectionNotify;
+  evnt_sel.xselection.requestor=event.xselectionrequest.requestor;
+  evnt_sel.xselection.selection=event.xselectionrequest.selection;
+  evnt_sel.xselection.target=Receiver;
+  evnt_sel.xselection.time=event.xselectionrequest.time;
+
+  if (event.xselectionrequest.target==Receiver)		/* On a trouve le recepteur */
+  {
+   evnt_sel.xselection.property=event.xselectionrequest.property;
+   XChangeProperty(x11base->display,evnt_sel.xselection.requestor,
+ 	       evnt_sel.xselection.property,
+  	       evnt_sel.xselection.target,
+  	       8,PropModeReplace,BuffSend.TabMsg[i].Msg,strlen(BuffSend.TabMsg[i].Msg)+1);
+   BuffSend.NbMsg--;
+   free(BuffSend.TabMsg[i].Msg);
+   if (BuffSend.NbMsg>0)
+   {
+    memmove(&BuffSend.TabMsg[i],&BuffSend.TabMsg[i+1],(BuffSend.NbMsg-i)*sizeof(TypeName));
+   }
+  }
+  else
+  {		/* Cas ou le recepteur demande un message et qu'il n'y en a pas */
+   evnt_sel.xselection.property=None;
+  }
+  XSendEvent(x11base->display,evnt_sel.xselection.requestor,False,0,&evnt_sel);
+ } 
+}
 
 /* read an X event */
 void ReadXServer ()
 {
  static XEvent event,evnt_sel;
  int i;
- KeySym keysym;
- char *octet,*nom_buffer;
+ char *octet;
  
   while (XEventsQueued(x11base->display, QueuedAfterReading))
   {
@@ -450,7 +528,7 @@ void ReadXServer ()
 	   {
 	    /* Envoi de l'evt  l'objet */
 	    for (i=0;i<nbobj;i++)
-	     if (tabxobj[i]->ObjWin==event.xkey.subwindow)
+	     if (tabxobj[i]->win==event.xkey.subwindow)
 	      tabxobj[i]->EvtKey(tabxobj[i],&event.xkey);
 	   }
 	  break;
@@ -459,7 +537,7 @@ void ReadXServer ()
 	  if (event.xbutton.subwindow!=0)
 	  {
 	   i=0;
-	   while ((tabxobj[i]->ObjWin!=event.xbutton.subwindow)&&(i<nbobj-1))
+	   while ((tabxobj[i]->win!=event.xbutton.subwindow)&&(i<nbobj-1))
 	    i++;
 	   tabxobj[i]->EvtMouse(tabxobj[i],&event.xbutton);
 	  }
@@ -496,7 +574,13 @@ void ReadXServer ()
             }
             XSendEvent(x11base->display,evnt_sel.xselection.requestor,False,0,&evnt_sel);
            }
+	   else
+	    SendMsgToScript(event);
            break;
+      case SelectionClear:
+	    if (event.xselectionclear.selection==XA_PRIMARY)
+	     UnselectAllTextField(tabxobj);
+            break;
       case ClientMessage:
 	   if ((event.xclient.format==32) && (event.xclient.data.l[0]==wm_del_win))
 	    DeadPipe(1);
@@ -520,7 +604,10 @@ void ReadXServer ()
             octet=XFetchBuffer(x11base->display,&i,7);
            else break;
            if (i>0)
+	   {
+	    Scrapt=(char*)realloc((void*)Scrapt,sizeof(char)*(i+1));
             Scrapt=strcpy(Scrapt,octet);
+	   }
           break;
     }
   }
@@ -533,42 +620,89 @@ void MainLoop ()
  unsigned long header[HEADER_SIZE];
  unsigned long *body;
  int count,i;
+ struct timeval tv;
+ int res;
 
-  while (1) {
-    FD_ZERO(&in_fdset);
-    FD_SET(x_fd,&in_fdset);
-    FD_SET(fd[1],&in_fdset);
+ while (1)
+ {
+  FD_ZERO(&in_fdset);
+  FD_SET(x_fd,&in_fdset);
+  FD_SET(fd[1],&in_fdset);
 
-    XFlush(x11base->display);
-    
-    if (select(32, &in_fdset, NULL, NULL, NULL) > 0)
+  XFlush(x11base->display);
+
+  tv.tv_sec = 1;
+  tv.tv_usec = 0;
+
+  if (x11base->periodictasks!=NULL)
+   res=select(32, &in_fdset, NULL, NULL, &tv);
+  else
+   res=select(32, &in_fdset, NULL, NULL, NULL);
+
+  if (res > 0)
+  {
+   if (FD_ISSET(x_fd, &in_fdset))
+    ReadXServer();
+
+   if(FD_ISSET(fd[1], &in_fdset))
+   {
+    if((count = ReadFvwmPacket(fd[1], header, &body)) > 0)
     {
-     if (FD_ISSET(x_fd, &in_fdset))
-      ReadXServer();
-
-     if(FD_ISSET(fd[1], &in_fdset))
-     {
-      if(count = ReadFvwmPacket(fd[1], header, &body) > 0)
-      {
-       for (i=0;i<nbobj;i++)
-        tabxobj[i]->ProcessMsg(tabxobj[i],header[1],body);
-       free(body);
-      }
-     }
-
+     for (i=0;i<nbobj;i++)
+      tabxobj[i]->ProcessMsg(tabxobj[i],header[1],body);
+     free(body);
     }
+   }
   }
+  if (x11base->periodictasks!=NULL)		/* Execution des taches periodics */
+   ExecBloc(x11base->periodictasks);
+ }
 }
     
+void ReadFvwmScriptArg(int argc, char **argv,int IsFather)
+{
+ int i;
+ Atom myatom;
+ int FisrtArg;
+
+ BuffSend.NbMsg=0;			/* Aucun message dans le buffer */
+ 
+ for (i=2;i<98;i++)
+  x11base->TabScriptId[i]=NULL;
+
+ if (IsFather)			/* Cas du pere */
+ {
+  myatom=XInternAtom(x11base->display,x11base->TabScriptId[1],True);
+  XSetSelectionOwner(x11base->display,myatom,x11base->win,CurrentTime);
+  FisrtArg=9;
+ }
+ else
+ {				/* Cas du fils */
+  x11base->TabScriptId[0]=(char*)calloc(sizeof(char),strlen(argv[7]));
+  x11base->TabScriptId[0]=strncpy(x11base->TabScriptId[0],argv[7],strlen("FvwmScript")+3);
+  x11base->TabScriptId[1]=argv[7];
+  myatom=XInternAtom(x11base->display,x11base->TabScriptId[1],True);
+  XSetSelectionOwner(x11base->display,myatom,x11base->win,CurrentTime);
+  FisrtArg=8;
+ }
+}
 
 /* main procedure */
-main (int argc, char **argv)
+void main (int argc, char **argv)
 {
+  int IsFather;
   int i;
 
   /* we get rid of the path from program name */
-  ScriptName = argv[0];
-  
+  ModuleName = argv[0];
+
+  /* On determine si le script a un pere */
+  if (argc>=8)
+   IsFather=(argv[7][0]!=(char)161);
+  else
+   IsFather=1;
+    
+
   signal (SIGPIPE, DeadPipe);  
   signal (SIGINT, DeadPipe);  /* cleanup on other ways of closing too */
   signal (SIGHUP, DeadPipe);  
@@ -577,11 +711,11 @@ main (int argc, char **argv)
 
   if (argc < 6)
   {
-    fprintf(stderr,"%s must be started by Fvwm.\n", ScriptName);
+    fprintf(stderr,"%s must be started by Fvwm.\n", ModuleName);
     exit(1);
   }
  else
-  if(argc==7)
+  if(argc>=7)
   {
    ScriptName = argv[6];
    ref = strtol(argv[4], NULL, 16);
@@ -591,10 +725,17 @@ main (int argc, char **argv)
    SetMessageMask(fd, M_NEW_DESK | M_END_WINDOWLIST| 
 		 M_MAP|  M_RES_NAME| M_RES_CLASS| M_CONFIG_INFO|
 		 M_END_CONFIG_INFO| M_WINDOW_NAME);
+
+   /* Enregistrement des arguments du script */
+   x11base=(X11base*) calloc(1,sizeof(X11base));
+   x11base->TabArg[0]=ModuleName;
+   for (i=8-IsFather;i<argc;i++)
+    x11base->TabArg[i-7+IsFather]=argv[i];
+
   }
   else 
   {
-    fprintf(stderr,"%s requires only the path of the script.\n", ScriptName);
+    fprintf(stderr,"%s requires only the path of the script.\n", ModuleName);
     exit(1);
   }
 
@@ -606,20 +747,10 @@ main (int argc, char **argv)
 
  InitCom();			/* Fonction d'initialisation de TabCom et TabFunc   */
 
- BuildGUI();			/* Construction des boutons et de la fenetre */
+ BuildGUI(IsFather);			/* Construction des boutons et de la fenetre */
+
+ ReadFvwmScriptArg(argc,argv,IsFather);
 
  MainLoop();
 
 }
-
-/* Gestion des erreurs de syntaxes */
-int yyerror(char *errmsg)
-{
- fprintf(stderr,"Line %d: %s\n",numligne,errmsg);
- return 0;
-}
-
-
-
-
-
