@@ -531,22 +531,6 @@ FvwmWindow *AddWindow(Window w)
 	  else
 	    tmp_win->right_w[i] = None;
 	}
-      
-#ifdef MINI_ICONS
-      if (tmp_win->mini_pixmap_file) {
-	tmp_win->mini_icon = CachePicture (dpy, Scr.Root, 
-					   IconPath,
-#ifdef XPM
-					   PixmapPath,
-#else
-					   NULL,
-#endif
-					   tmp_win->mini_pixmap_file);
-      }
-      else {
-	tmp_win->mini_icon = NULL;
-      }
-#endif
     }
 
   if(tmp_win->flags & BORDER)
@@ -577,6 +561,23 @@ FvwmWindow *AddWindow(Window w)
 #endif
     }
 
+      
+#ifdef MINI_ICONS
+  if (tmp_win->mini_pixmap_file) {
+    tmp_win->mini_icon = CachePicture (dpy, Scr.Root, 
+                                       IconPath,
+#ifdef XPM
+                                       PixmapPath,
+#else
+                                       NULL,
+#endif
+                                       tmp_win->mini_pixmap_file);
+  }
+  else {
+    tmp_win->mini_icon = NULL;
+  }
+#endif
+
   XMapSubwindows (dpy, tmp_win->frame);
   XRaiseWindow(dpy,tmp_win->Parent);
   XReparentWindow(dpy, tmp_win->w, tmp_win->Parent,0,0);
@@ -648,10 +649,8 @@ FvwmWindow *AddWindow(Window w)
 			&a,&b,&JunkChild);
   tmp_win->xdiff -= a;
   tmp_win->ydiff -= b;
-#ifndef CLICKTORAISE
-  if(tmp_win->flags & ClickToFocus)
+  if((tmp_win->flags & ClickToFocus) || Scr.MouseFocusClickRaises)
     {
-#endif
      /* need to grab all buttons for window that we are about to
        * unhighlight */
       for(i=0;i<3;i++)
@@ -664,9 +663,7 @@ FvwmWindow *AddWindow(Window w)
 			ButtonPressMask, GrabModeSync,GrabModeAsync,None,
 			Scr.FvwmCursors[SYS]);
 	  }
-#ifndef CLICKTORAISE
     }
-#endif
   BroadcastConfig(M_ADD_WINDOW,tmp_win);
 
   BroadcastName(M_WINDOW_NAME,tmp_win->w,tmp_win->frame,
@@ -683,13 +680,14 @@ FvwmWindow *AddWindow(Window w)
 		(unsigned long)tmp_win,tmp_win->class.res_name);
 #ifdef MINI_ICONS
   if (tmp_win->mini_icon != NULL)
-    Broadcast(M_MINI_ICON, 6,
-              tmp_win->w, /* Watch Out ! : I reduced the set of infos... */
-              tmp_win->mini_icon->picture,
-              tmp_win->mini_icon->mask,
-              tmp_win->mini_icon->width,
-              tmp_win->mini_icon->height,
-              tmp_win->mini_icon->depth, 0);
+    BroadcastMiniIcon(M_MINI_ICON,
+                      tmp_win->w, tmp_win->frame, (unsigned long)tmp_win,
+                      tmp_win->mini_icon->width,
+                      tmp_win->mini_icon->height,
+                      tmp_win->mini_icon->depth,
+                      tmp_win->mini_icon->picture,
+                      tmp_win->mini_icon->mask,
+                      tmp_win->mini_pixmap_file);
 #endif
 
   FetchWmProtocols (tmp_win);
