@@ -586,7 +586,7 @@ void DrawButton(FvwmWindow *t, Window win, int w, int h,
 #ifdef VECTOR_BUTTONS
     case VectorButton:
 	if((t->flags & MWMButtons)
-	   && (stateflags & MWMButton)
+           && (stateflags & MWMDecorMaximize)
 	   && (t->flags & MAXIMIZED))
 	    DrawLinePattern(win,
 			    ShadowGC, ReliefGC,
@@ -1320,6 +1320,12 @@ void SetupFrame(FvwmWindow *tmp_win,int x,int y,int w,int h,Bool sendEvent)
   int shaded = tmp_win->buttons & WSHADE;
 #endif
 
+#ifdef FVWM_DEBUG_MSGS
+  fvwm_msg(DBG,"SetupFrame",
+           "Routine Entered (x == %d, y == %d, w == %d, h == %d)",
+           x, y, w, h);
+#endif
+
   /* if windows is not being maximized, save size in case of maximization */
   if (!(tmp_win->flags & MAXIMIZED) 
 #ifdef WINDOWSHADE
@@ -1529,7 +1535,11 @@ void SetupFrame(FvwmWindow *tmp_win,int x,int y,int w,int h,Bool sendEvent)
   frame_wc.height = tmp_win->frame_height = h;
   frame_mask = (CWX | CWY | CWWidth | CWHeight);
   XConfigureWindow (dpy, tmp_win->frame, frame_mask, &frame_wc);
-
+#ifdef FVWM_DEBUG_MSGS
+  fvwm_msg(DBG,"SetupFrame",
+           "New frame dimensions (x == %d, y == %d, w == %d, h == %d)",
+           frame_wc.x, frame_wc.y, frame_wc.width, frame_wc.height);
+#endif
 
 #ifdef SHAPE
   if (ShapesSupported)
@@ -1565,7 +1575,12 @@ void SetupFrame(FvwmWindow *tmp_win,int x,int y,int w,int h,Bool sendEvent)
     client_event.xconfigure.above = tmp_win->frame;
     client_event.xconfigure.override_redirect = False;
     XSendEvent(dpy, tmp_win->w, False, StructureNotifyMask, &client_event);
+#ifdef FVWM_DEBUG_MSGS
+    fvwm_msg(DBG,"SetupFrame","Sent ConfigureNotify (w == %d, h == %d)",
+             client_event.xconfigure.width,client_event.xconfigure.height);
+#endif
   }
+  XSync(dpy,0);
 
   BroadcastConfig(M_CONFIGURE_WINDOW,tmp_win);
 }
