@@ -211,7 +211,9 @@ void FocusOn(FvwmWindow *t,int DeIconifyOnly)
   if(!(t->flags & ClickToFocus))
     XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x+2,y+2);
 #endif /* 0 */
+#if 0 /* don't want to raise anymore either */
   RaiseWindow(t);
+#endif /* 0 */
   KeepOnTop();
 
   /* If the window is still not visible, make it visible! */
@@ -407,7 +409,8 @@ void WindowShade(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		   tmp_win->orig_wd,
 		   tmp_win->orig_ht,
 		   True);
-        Broadcast(M_DEWINDOWSHADE, 1, tmp_win->w, 0, 0, 0, 0, 0, 0);
+        Broadcast(M_DEWINDOWSHADE, 3, tmp_win->w, tmp_win->frame,
+                  (unsigned long)tmp_win, 0, 0, 0, 0);
     }
     else
     {
@@ -418,7 +421,8 @@ void WindowShade(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		   tmp_win->frame_width,
 		   tmp_win->title_height + tmp_win->boundary_width,
 		   False);
-        Broadcast(M_WINDOWSHADE, 1, tmp_win->w, 0, 0, 0, 0, 0, 0);
+        Broadcast(M_WINDOWSHADE, 3, tmp_win->w, tmp_win->frame,
+                  (unsigned long)tmp_win, 0, 0, 0, 0);
     }
 }
 #endif /* WINDOWSHADE */
@@ -667,7 +671,7 @@ void iconify_function(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		      unsigned long context,char *action, int *Module)
 
 {
-  long val1;
+  int val1;
   int val1_unit,n;
 
   if (DeferExecution(eventp,&w,&tmp_win,&context, SELECT, 
@@ -977,25 +981,6 @@ void wait_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 }
 
 
-void raise_it_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
-		   unsigned long context, char *action, int *Module)
-{
-  int val1_unit,n;
-  long val1; 
-  n = GetOneArgument(action, &val1, &val1_unit);
-
-  if(val1 != 0)
-  {
-    FocusOn((FvwmWindow *)val1,0);
-    if (((FvwmWindow *)(val1))->flags & ICONIFIED)
-    {
-      DeIconify((FvwmWindow *)val1);
-      FocusOn((FvwmWindow *)val1,0);
-    }
-  }
-}
-      
-
 void flip_focus_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		unsigned long context, char *action, int *Module)
 {
@@ -1134,6 +1119,9 @@ void echo_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 {
   if (action && *action)
   {
+    int len=strlen(action);
+    if (action[len-1]=='\n')
+      action[len-1]='\0';
     fvwm_msg(INFO,"Echo",action);
   }
 }
@@ -1252,7 +1240,7 @@ void SetColormapFocus(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 void SetClick(XEvent *eventp,Window w,FvwmWindow *tmp_win,
               unsigned long context, char *action,int* Module)
 {
-  long val1;
+  int val1;
   int val1_unit,n;
 
   n = GetOneArgument(action, &val1, &val1_unit);
@@ -1268,7 +1256,7 @@ void SetClick(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 void SetXOR(XEvent *eventp,Window w,FvwmWindow *tmp_win,
             unsigned long context, char *action,int* Module)
 {
-  long val1;
+  int val1;
   int val1_unit,n;
   XGCValues gcv;
   unsigned long gcm;
@@ -1280,10 +1268,14 @@ void SetXOR(XEvent *eventp,Window w,FvwmWindow *tmp_win,
     return;
   }
 
-  gcm = GCFunction|GCLineWidth|GCForeground|GCSubwindowMode; 
+  gcm = GCFunction|GCLineWidth|GCForeground|GCSubwindowMode;
   gcv.function = GXxor;
   gcv.line_width = 0;
-  gcv.foreground = val1;
+  /* use passed in value, or try to calculate appropriate value if 0 */
+  /* ctwm method: */
+  /* gcv.foreground = (val1)?(val1):((((unsigned long) 1) << Scr.d_depth) - 1); */
+  /* Xlib programming manual suggestion: */
+  gcv.foreground = (val1)?(val1):(BlackPixel(dpy,Scr.screen) ^ WhitePixel(dpy,Scr.screen));
   gcv.subwindow_mode = IncludeInferiors;
   Scr.DrawGC = XCreateGC(dpy, Scr.Root, gcm, &gcv);
 }
@@ -1291,7 +1283,7 @@ void SetXOR(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 void SetOpaque(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 	       unsigned long context, char *action,int* Module)
 {
-  long val1;
+  int val1;
   int val1_unit,n;
 
   n = GetOneArgument(action, &val1, &val1_unit);
@@ -1463,9 +1455,126 @@ void SetHiColor(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 }
 
 
+void SafeDefineCursor(Window w, Cursor cursor)
+{
+  if (w) XDefineCursor(dpy,w,cursor);
+}
+
+void CursorStyle(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
+                 unsigned long context, char *action,int* Module)
+{
+  char *cname=NULL, *newcursor=NULL;
+  int index,nc,i;
+  FvwmWindow *fw;
+  MenuRoot *mr;
+
+  action = GetNextToken(action,&cname);
+  action = GetNextToken(action,&newcursor);
+  if (!cname || !newcursor)
+  {
+    fvwm_msg(ERR,"CursorStyle","Bad cursor style");
+    return;
+  }
+  if (StrEquals("POSITION",cname)) index = POSITION;
+  else if (StrEquals("DEFAULT",cname)) index = DEFAULT;
+  else if (StrEquals("SYS",cname)) index = SYS;
+  else if (StrEquals("TITLE",cname)) index = TITLE_CURSOR;
+  else if (StrEquals("MOVE",cname)) index = MOVE;
+  else if (StrEquals("MENU",cname)) index = MENU;
+  else if (StrEquals("WAIT",cname)) index = WAIT;
+  else if (StrEquals("SELECT",cname)) index = SELECT;
+  else if (StrEquals("DESTROY",cname)) index = DESTROY;
+  else if (StrEquals("LEFT",cname)) index = LEFT;
+  else if (StrEquals("RIGHT",cname)) index = RIGHT;
+  else if (StrEquals("TOP",cname)) index = TOP;
+  else if (StrEquals("BOTTOM",cname)) index = BOTTOM;
+  else if (StrEquals("TOP_LEFT",cname)) index = TOP_LEFT;
+  else if (StrEquals("TOP_RIGHT",cname)) index = TOP_RIGHT;
+  else if (StrEquals("BOTTOM_LEFT",cname)) index = BOTTOM_LEFT;
+  else if (StrEquals("BOTTOM_RIGHT",cname)) index = BOTTOM_RIGHT;
+  else
+  {
+    fvwm_msg(ERR,"CursorStyle","Unknown cursor name %s",cname);
+    return;
+  }
+  nc = atoi(newcursor);
+  if ((nc < 0) || (nc >= XC_num_glyphs) || ((nc % 2) != 0))
+  {
+    fvwm_msg(ERR,"CursorStyle","Bad cursor number %s",newcursor);
+    return;
+  }
+
+  /* replace the cursor defn */
+  if (Scr.FvwmCursors[index]) XFreeCursor(dpy,Scr.FvwmCursors[index]);
+  Scr.FvwmCursors[index] = XCreateFontCursor(dpy,nc);
+
+  /* redefine all the windows using cursors */
+  fw = Scr.FvwmRoot.next;
+  while(fw != NULL)
+  {
+    for (i=0;i<4;i++)
+    {
+      SafeDefineCursor(fw->corners[i],Scr.FvwmCursors[TOP_LEFT+i]);
+      SafeDefineCursor(fw->sides[i],Scr.FvwmCursors[TOP+i]);
+    }
+    for (i=0;i<Scr.nr_left_buttons;i++)
+    {
+      SafeDefineCursor(fw->left_w[i],Scr.FvwmCursors[SYS]);
+    }
+    for (i=0;i<Scr.nr_right_buttons;i++)
+    {
+      SafeDefineCursor(fw->right_w[i],Scr.FvwmCursors[SYS]);
+    }
+    SafeDefineCursor(fw->title_w, Scr.FvwmCursors[TITLE_CURSOR]);      
+    fw = fw->next;
+  }
+
+  /* Do the menus for good measure */
+  mr = Scr.AllMenus;
+  while(mr != NULL)
+  {
+    SafeDefineCursor(mr->w,Scr.FvwmCursors[MENU]);
+    mr = mr->next;
+  }
+}
+
 void SetMenuStyle(XEvent *eventp,Window w,FvwmWindow *tmp_win,
                   unsigned long context, char *action,int* Module)
 {
+#if 0 /* new parse syntax -- unfinished */
+  if (action && *action)
+  {
+    line = strdup(action);
+    while (line && *line)
+    {
+      tok = GetToken(&line);
+      if (StrEquals(tok,"Color"))
+      {
+        /* Color fg/bg/st */
+      }
+      else if (StrEquals(tok,"mwm"))
+      {
+        Scr.flags |= MWMMenus;
+      }
+      else if (StrEquals(tok,"fvwm"))
+      {
+        Scr.flags &= ~MWMMenus;
+      }
+      else if (StrEquals(tok,"Font"))
+      {
+      }
+      else if (StrEquals(tok,"NoWarp"))
+      {
+      }
+      else if (StrEquals(tok,"Warp"))
+      {
+      }
+      else if (StrEquals(tok,"Warp"))
+      {
+      }
+    }
+  }
+#else
   XGCValues gcv;
   unsigned long gcm;
   char *fore=NULL, *back=NULL, *stipple = NULL, *font= NULL, *style = 0;
@@ -1604,7 +1713,7 @@ void SetMenuStyle(XEvent *eventp,Window w,FvwmWindow *tmp_win,
     free(font);
   if(style != NULL)
     free(style);
-
+#endif /* 0 */
 }
 
 Boolean ReadButtonFace(char *s, ButtonFace *bf, int button, int verbose);
@@ -2608,6 +2717,28 @@ void UpdateDecor(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
  * Changes a button decoration style (changes by veliaa@rpi.edu)
  *
  ****************************************************************************/
+#define SetButtonFlag(a) \
+        do { \
+             int i; \
+             if (multi) { \
+               if (multi&1) \
+                 for (i=0;i<5;++i) \
+                   if (set) \
+                     fl->left_buttons[i].flags |= (a); \
+                   else \
+                     fl->left_buttons[i].flags &= ~(a); \
+               if (multi&2) \
+                 for (i=0;i<5;++i) \
+                   if (set) \
+                     fl->right_buttons[i].flags |= (a); \
+                   else \
+                     fl->right_buttons[i].flags &= ~(a); \
+             } else \
+                 if (set) \
+                   tb->flags |= (a); \
+                 else \
+                   tb->flags &= ~(a); } while (0)
+
 void ButtonStyle(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
                  unsigned long context, char *action,int* Module)
 {
@@ -2705,14 +2836,12 @@ void ButtonStyle(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
 			else
 			    tb->flags = ~0;
 		}
-		else if (mystrncasecmp(tok,"MWMButton",9)==0)
-		{
-		    if (!multi)
-			if (set)
-			    tb->flags |= MWMButton;
-			else
-			    tb->flags &= ~MWMButton;
-		}
+                else if (mystrncasecmp(tok,"MWMDecorMenu",12)==0)
+                  SetButtonFlag(MWMDecorMenu);
+                else if (mystrncasecmp(tok,"MWMDecorMin",11)==0)
+                  SetButtonFlag(MWMDecorMinimize);
+                else if (mystrncasecmp(tok,"MWMDecorMax",11)==0)
+                  SetButtonFlag(MWMDecorMaximize);
 		else
 		    fvwm_msg(ERR,"ButtonStyle",
 			     "unknown title button flag %s -- line: %s", tok, text);
@@ -2839,6 +2968,7 @@ void AddButtonStyle(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
  *
  * Direction = 1 ==> "Next" operation
  * Direction = -1 ==> "Previous" operation 
+ * Direction = 0 ==> operation on current window (returns pass or fail)
  *
  **************************************************************************/
 FvwmWindow *Circulate(char *action, int Direction, char **restofline)
@@ -2847,15 +2977,15 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
   FvwmWindow *fw, *found = NULL;
   char *t,*tstart,*name = NULL, *expression, *condition, *prev_condition=NULL;
   char *orig_expr;
-  Bool needsIconic = 0;
-  Bool needsNormal = 0;
   Bool needsCurrentDesk = 0;
   Bool needsCurrentPage = 0;
-  Bool needsVisible = 0;
-  Bool needsInvisible = 0;
-  char *AnyWindow = "*";
-  Bool useCirculateHit = 0;
-  Bool useCirculateHitIcon = 0;
+
+  Bool needsName = 0;
+  Bool needsNotName = 0;
+  Bool useCirculateHit = (Direction) ? 0 : 1; /* override for Current [] */
+  Bool useCirculateHitIcon = (Direction) ? 0 : 1;
+  unsigned long onFlags = 0;
+  unsigned long offFlags = 0;
 
   l=0;
 
@@ -2889,10 +3019,30 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
     expression = GetNextToken(expression,&condition);
     while((condition != NULL)&&(strlen(condition) > 0))
     {
-      if(mystrcasecmp(condition,"iconic")==0)
-        needsIconic = 1;
-      else if(mystrcasecmp(condition,"!iconic")==0)
-        needsNormal = 1;
+      if     (mystrcasecmp(condition,"Iconic")==0)
+        onFlags |= ICONIFIED;
+      else if(mystrcasecmp(condition,"!Iconic")==0)
+        offFlags |= ICONIFIED;
+      else if(mystrcasecmp(condition,"Visible")==0)
+        onFlags |= VISIBLE;
+      else if(mystrcasecmp(condition,"!Visible")==0)
+        offFlags |= VISIBLE;
+      else if(mystrcasecmp(condition,"Sticky")==0)
+        onFlags |= STICKY;
+      else if(mystrcasecmp(condition,"!Sticky")==0)
+        offFlags |= STICKY;
+      else if(mystrcasecmp(condition,"Maximized")==0)
+        onFlags |= MAXIMIZED;
+      else if(mystrcasecmp(condition,"!Maximized")==0)
+        offFlags |= MAXIMIZED;
+      else if(mystrcasecmp(condition,"Transient")==0)
+        onFlags |= TRANSIENT;
+      else if(mystrcasecmp(condition,"!Transient")==0)
+        offFlags |= TRANSIENT;
+      else if(mystrcasecmp(condition,"Raised")==0)
+        onFlags |= RAISED;
+      else if(mystrcasecmp(condition,"!Raised")==0)
+        offFlags |= RAISED;
       else if(mystrcasecmp(condition,"CurrentDesk")==0)
         needsCurrentDesk = 1;
       else if(mystrcasecmp(condition,"CurrentPage")==0)
@@ -2900,10 +3050,6 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
         needsCurrentDesk = 1;
         needsCurrentPage = 1;
       }
-      else if(mystrcasecmp(condition,"Visible")==0)
-        needsVisible = 1;
-      else if(mystrcasecmp(condition,"!Visible")==0)
-        needsInvisible = 1;
       else if(mystrcasecmp(condition,"CurrentPageAnyDesk")==0 ||
               mystrcasecmp(condition,"CurrentScreen")==0)
         needsCurrentPage = 1;
@@ -2911,10 +3057,17 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
         useCirculateHit = 1;
       else if(mystrcasecmp(condition,"CirculateHitIcon")==0)
         useCirculateHitIcon = 1;
-      else
+      else if(!needsName && !needsNotName) /* only 1st name to avoid mem leak */
       {
         name = condition;
         condition = NULL;
+        if (name[0] == '!')
+        {
+          needsNotName = 1;
+          name++;
+        }
+        else
+          needsName = 1;
       }
       if(prev_condition)free(prev_condition);
       prev_condition = condition;
@@ -2928,15 +3081,14 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
   else
     *restofline = t;
 
-  if(name == NULL)
-    name = AnyWindow;
-
   if(Scr.Focus != NULL)
   {
     if(Direction == 1)
       fw = Scr.Focus->prev;
-    else
+    else if(Direction == -1)
       fw = Scr.Focus->next;
+    else
+      fw = Scr.Focus;
   }
   else
     fw = Scr.FvwmRoot.prev;  
@@ -2946,24 +3098,31 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
     while((fw != NULL)&&(found==NULL)&&(fw != &Scr.FvwmRoot))
     {
       /* Make CirculateUp and CirculateDown take args. by Y.NOMURA */
-      if (((matchWildcards(name, fw->name)) ||
-           (matchWildcards(name, fw->icon_name))||
-           (fw->class.res_class &&
-            matchWildcards(name, fw->class.res_class))||
-           (fw->class.res_name &&
-            matchWildcards(name, fw->class.res_name)))&&
-          ((useCirculateHit)||!(fw->flags & CirculateSkip))&&
-          (((useCirculateHitIcon)&&(fw->flags & ICONIFIED))||
-           !((fw->flags & CirculateSkipIcon)&&(fw->flags & ICONIFIED)))&&
-          (((!needsIconic)||(fw->flags & ICONIFIED))&&
-           ((!needsNormal)||(!(fw->flags & ICONIFIED)))&&
-           ((!needsCurrentDesk)||(fw->Desk == Scr.CurrentDesk))&&
-           ((!needsVisible)||(fw->flags & VISIBLE))&&
-           ((!needsInvisible)||(!(fw->flags & VISIBLE)))&&
-           ((!needsCurrentPage)||((fw->frame_x < Scr.MyDisplayWidth)&&
-                                  (fw->frame_y < Scr.MyDisplayHeight)&&
-                                  (fw->frame_x+fw->frame_width > 0)&&
-                                  (fw->frame_y+fw->frame_height > 0)))))
+      if ((onFlags & fw->flags) == onFlags &&
+          (offFlags & fw->flags) == 0 &&
+          (useCirculateHit || !(fw->flags & CirculateSkip)) &&
+          ((useCirculateHitIcon && fw->flags & ICONIFIED) ||
+           !(fw->flags & CirculateSkipIcon && fw->flags & ICONIFIED)) &&
+          (!needsCurrentDesk || fw->Desk == Scr.CurrentDesk) &&
+          (!needsCurrentPage || (fw->frame_x < Scr.MyDisplayWidth &&
+                                 fw->frame_y < Scr.MyDisplayHeight &&
+                                 fw->frame_x + fw->frame_width > 0 &&
+                                 fw->frame_y + fw->frame_height > 0)
+            ) &&
+          (!needsName ||
+           matchWildcards(name, fw->name) ||
+           matchWildcards(name, fw->icon_name) ||
+           fw->class.res_class && matchWildcards(name, fw->class.res_class) ||
+           fw->class.res_name && matchWildcards(name, fw->class.res_name)
+            ) &&
+          (!needsNotName ||
+           !(matchWildcards(name, fw->name) ||
+             matchWildcards(name, fw->icon_name) ||
+             fw->class.res_class && matchWildcards(name, fw->class.res_class) ||
+             fw->class.res_name && matchWildcards(name, fw->class.res_name)
+             )
+            )
+        )
         found = fw;
       else
       {
@@ -2972,6 +3131,14 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
         else
           fw = fw->next;
       }
+      if (Direction == 0)
+      {
+        if (needsName)
+          free(name);
+        else if (needsNotName)
+          free(name - 1);
+        return found;
+      }
     }
     if((fw == NULL)||(fw == &Scr.FvwmRoot))
     {
@@ -2992,8 +3159,10 @@ FvwmWindow *Circulate(char *action, int Direction, char **restofline)
     }
     pass++;
   }
-  if((name != NULL)&&(name != AnyWindow))
+  if (needsName)
     free(name);
+  else if (needsNotName)
+    free(name - 1);
   return found;
 
 }
@@ -3026,7 +3195,6 @@ void NextFunc(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
 
 }
 
-
 void NoneFunc(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
               unsigned long context, char *action,int* Module)
 {
@@ -3040,6 +3208,19 @@ void NoneFunc(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
   }
 }
 
+void CurrentFunc(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
+              unsigned long context, char *action,int* Module)
+{
+  FvwmWindow *found;
+  char *restofline;
+
+  found = Circulate(action, 0, &restofline);
+  if(found != NULL)
+  {
+    ExecuteFunction(restofline,found,eventp,C_WINDOW,*Module);
+  }
+}
+
 void WindowIdFunc(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
                   unsigned long context, char *action,int* Module)
 {
