@@ -379,19 +379,40 @@ void SelectDecor(FvwmWindow *t, unsigned long tflags, int border_width,
     {
       /*  title-bar menu button omitted 
        * window gets 1 pixel wide black border */
-      t->left_w[0] = None;
+      /* disable any buttons with the MWMDecorMenu flag */
+      int i;
+      for (i = 0; i < 5; ++i) {
+        if (GetDecor(t,left_buttons[i].flags)&MWMDecorMenu)
+          t->left_w[i] = None;
+        if (GetDecor(t,right_buttons[i].flags)&MWMDecorMenu)
+          t->right_w[i] = None;
+      }
     }
   if(!(decor & MWM_DECOR_MINIMIZE))
     {
       /* title-bar + iconify button, no menu button.
        * window gets 1 pixel wide black border */
-      t->right_w[1] = None;
+      /* disable any buttons with the MWMDecorMinimize flag */
+      int i;
+      for (i = 0; i < 5; ++i) {
+        if (GetDecor(t,left_buttons[i].flags)&MWMDecorMinimize)
+          t->left_w[i] = None;
+        if (GetDecor(t,right_buttons[i].flags)&MWMDecorMinimize)
+          t->right_w[i] = None;
+      }
     }
   if(!(decor & MWM_DECOR_MAXIMIZE))
     {
       /* title-bar + maximize button, no menu button, no iconify.
        * window has 1 pixel wide black border */
-      t->right_w[0] = None;
+      /* disable any buttons with the MWMDecorMaximize flag */
+      int i;
+      for (i = 0; i < 5; ++i) {
+        if (GetDecor(t,left_buttons[i].flags)&MWMDecorMaximize)
+          t->left_w[i] = None;
+        if (GetDecor(t,right_buttons[i].flags)&MWMDecorMaximize)
+          t->right_w[i] = None;
+      }
     }
   if (t->buttons & BUTTON1) t->left_w[0]=None;
   if (t->buttons & BUTTON3) t->left_w[1]=None;
