@@ -513,6 +513,12 @@ void HandlePropertyNotify()
 	int new_width, new_height;
 	
 	GetWindowSizeHints (Tmp_win);
+        /*
+        ** ckh - not sure why this next stuff was here, but fvwm 1.xx
+        ** didn't do this, and it seems to cause a bug when changing
+        ** fonts in XTerm
+        */
+#if 0
 	new_width = Tmp_win->frame_width;
 	new_height = Tmp_win->frame_height;
 	ConstrainSize(Tmp_win, &new_width, &new_height);
@@ -520,7 +526,7 @@ void HandlePropertyNotify()
 	   (new_height != Tmp_win->frame_height))
 	  SetupFrame(Tmp_win,Tmp_win->frame_x, Tmp_win->frame_y,
 		     new_width,new_height,False);
-
+#endif /* 0 */
 	BroadcastConfig(M_CONFIGURE_WINDOW,Tmp_win);
       }
       break;
@@ -963,7 +969,7 @@ void HandleButtonPress()
   DBUG("HandleButtonPress","Routine Entered");
 
   /* click to focus stuff goes here */
-  if((Tmp_win)&&(Tmp_win->flags & ClickToFocus)&&(Tmp_win != Scr.Ungrabbed)&&
+  if((Tmp_win)&&(Tmp_win->flags & ClickToFocus)&&(Tmp_win != Scr.Ungrabbed) &&
      ((Event.xbutton.state&
        (ControlMask|Mod1Mask|Mod2Mask|Mod3Mask|Mod4Mask|Mod5Mask)) == 0))
   {
@@ -1087,29 +1093,30 @@ void HandleEnterNotify()
       || ewp->window==Scr.PanFrameLeft.win
       || ewp->window==Scr.PanFrameRight.win 
       || ewp->window==Scr.PanFrameBottom.win ) 
-    {
-      int delta_x=0, delta_y=0;
-      /* this was in the HandleMotionNotify before, HEDU */
-      HandlePaging(Scr.EdgeScrollX,Scr.EdgeScrollY,
-                   &Event.xcrossing.x_root,&Event.xcrossing.y_root,
-                   &delta_x,&delta_y,True);
-      return;
-    }
+  {
+    int delta_x=0, delta_y=0;
+    /* this was in the HandleMotionNotify before, HEDU */
+    HandlePaging(Scr.EdgeScrollX,Scr.EdgeScrollY,
+                 &Event.xcrossing.x_root,&Event.xcrossing.y_root,
+                 &delta_x,&delta_y,True);
+    return;
+  }
 #endif /* NON_VIRTUAL */
- 
-  if(Event.xany.window == Scr.Root)
+
+  /* multi screen? */
+  if (Event.xany.window == Scr.Root)
+  {
+    if (!Scr.Focus || (!(Scr.Focus->flags&ClickToFocus)&&
+                       !(Scr.Focus->flags&SloppyFocus)))
     {
-      if((Scr.Focus)&&(!(Scr.Focus->flags&ClickToFocus))&&
-	 (!(Scr.Focus->flags&SloppyFocus)))
-	{
-	  SetFocus(Scr.NoFocusWin,NULL,1);
-	}
-      if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
-	{
-	  InstallWindowColormaps(NULL);
-	}
-      return;
+      SetFocus(Scr.NoFocusWin,NULL,1);
     }
+    if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
+    {
+      InstallWindowColormaps(NULL);
+    }
+    return;
+  }
   
   /* make sure its for one of our windows */
   if (!Tmp_win) 
@@ -1117,12 +1124,16 @@ void HandleEnterNotify()
   
   if(!(Tmp_win->flags & ClickToFocus))
     {
+#if 0
       if(Scr.Focus != Tmp_win)
 	{
 	  SetFocus(Tmp_win->w,Tmp_win,0);
 	}
       else
 	SetFocus(Tmp_win->w,Tmp_win,0);
+#else
+      SetFocus(Tmp_win->w,Tmp_win,1);
+#endif
     }
   if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
   {
@@ -1294,6 +1305,7 @@ void HandleConfigureRequest()
    * requested client window width; the inner height is the same as the
    * requested client window height plus any title bar slop.
    */
+  ConstrainSize(Tmp_win, &width, &height);
   SetupFrame (Tmp_win, x, y, width, height,sendEvent);
   KeepOnTop();
   
