@@ -336,6 +336,8 @@ void SetEdgeScroll(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		     unsigned long context, char *action,int *Module);
 void SetEdgeResistance(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		     unsigned long context, char *action,int *Module);
+void CursorStyle(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
+                 unsigned long context, char *action,int* Module);
 void ButtonStyle(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		 unsigned long context, char *action,int *Module);
 #ifdef MULTISTYLE
@@ -424,8 +426,6 @@ void goto_page_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 
 void wait_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 	       unsigned long context,char *action, int *Module);
-void raise_it_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
-		   unsigned long context, char *action, int *Module);
 void flip_focus_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		unsigned long context, char *action, int *Module);
 void focus_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
@@ -522,6 +522,8 @@ void PrevFunc(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 	      unsigned long context, char *action,int* Module);
 void NoneFunc(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 	      unsigned long context, char *action,int* Module);
+void CurrentFunc(XEvent *eventp,Window w,FvwmWindow *tmp_win,
+ 	      unsigned long context, char *action,int* Module);
 void WindowIdFunc(XEvent *eventp,Window w,FvwmWindow *tmp_win,
                   unsigned long context, char *action,int* Module);
 void ReadFile(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
