@@ -533,6 +533,57 @@ void BroadcastName(unsigned long event_type, unsigned long data1,
     SendName(i,event_type,data1,data2,data3,name);
 }
 
+#ifdef MINI_ICONS
+void SendMiniIcon(int module, unsigned long event_type,
+		  unsigned long data1, unsigned long data2, 
+		  unsigned long data3, unsigned long data4,
+		  unsigned long data5, unsigned long data6,
+		  unsigned long data7, unsigned long data8,
+		  char *name)
+{
+  int l;
+  unsigned long *body;
+  extern Time lastTimestamp;
+
+  if ((name == NULL) || (event_type != M_MINI_ICON))
+    return;
+  l = strlen(name)/(sizeof(unsigned long))+HEADER_SIZE+9;
+  body = (unsigned long *)safemalloc(l*sizeof(unsigned long));
+
+  body[0] = START_FLAG;
+  body[1] = event_type;
+  body[2] = l;
+  body[3] = lastTimestamp;
+
+  body[HEADER_SIZE] = data1;
+  body[HEADER_SIZE+1] = data2;
+  body[HEADER_SIZE+2] = data3; 
+  body[HEADER_SIZE+3] = data4; 
+  body[HEADER_SIZE+4] = data5; 
+  body[HEADER_SIZE+5] = data6; 
+  body[HEADER_SIZE+6] = data7; 
+  body[HEADER_SIZE+7] = data8; 
+  strcpy((char *)&body[HEADER_SIZE+8],name);
+
+  PositiveWrite(module,(unsigned long *)body, l*sizeof(unsigned long));
+
+  free(body);
+}
+
+void BroadcastMiniIcon(unsigned long event_type,
+		       unsigned long data1, unsigned long data2, 
+		       unsigned long data3, unsigned long data4,
+		       unsigned long data5, unsigned long data6,
+		       unsigned long data7, unsigned long data8,
+		       char *name)
+{
+  int i;
+
+  for(i=0;i<npipes;i++)
+    SendMiniIcon(i, event_type,
+		 data1, data2, data3, data4, data5, data6, data7, data8, name);
+}
+#endif /* MINI_ICONS */
 
 /*
 ** send an arbitrary string to all instances of a module
@@ -696,13 +747,14 @@ void send_list_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		       (unsigned long)t,0,0,0,0);
 #ifdef MINI_ICONS
 	  if (t->mini_icon != NULL) 
-	    SendPacket(*Module, M_MINI_ICON, 6,
-                       t->w,
-                       t->mini_icon->picture,
-                       t->mini_icon->mask,
-                       t->mini_icon->width,
-                       t->mini_icon->height,
-                       t->mini_icon->depth, 0);
+            SendMiniIcon(*Module, M_MINI_ICON,
+                         t->w, t->frame, (unsigned long)t,
+                         t->mini_icon->width,
+                         t->mini_icon->height,
+                         t->mini_icon->depth,
+                         t->mini_icon->picture,
+                         t->mini_icon->mask,
+                         t->mini_pixmap_file);
 #endif
 	}
       if(Scr.Hilite == NULL)
@@ -725,7 +777,7 @@ void set_mask_function(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		     unsigned long context, char *action,int* Module)
 {
   int n, val1_unit;
-  long val1;
+  int val1;
   n = GetOneArgument(action, &val1, &val1_unit);
   PipeMask[*Module] = (unsigned long)val1;
 }
