@@ -115,7 +115,9 @@ void ProcessNewStyle(XEvent *eventp,
             restofline++;
           while(isspace(*restofline))restofline++;
 	  
-          on_buttons |= (1<<(butt-1));        
+          if (butt == 0) butt = 10;
+          if (butt > 0 && butt <= 10)
+            on_buttons |= (1<<(butt-1));
         }
         else if(mystrncasecmp(restofline,"BorderWidth",11)==0)
         {
@@ -293,6 +295,13 @@ void ProcessNewStyle(XEvent *eventp,
           for(i=0;i<num;i++)
           {
             while(isspace(*restofline))restofline++;
+            if (*restofline == '-') {   /* If leading minus sign */
+              if (i == 0 || i == 2) {   /* if a width */
+                IconBox[i] += Scr.MyDisplayWidth;
+              } else {                  /* it must be a height */
+                IconBox[i] += Scr.MyDisplayHeight;
+              } /* end width/height */
+            } /* end leading minus sign */
             while((!isspace(*restofline))&&(*restofline!= 0)&&
                   (*restofline != ',')&&(*restofline != '\n'))
               restofline++;
@@ -300,18 +309,6 @@ void ProcessNewStyle(XEvent *eventp,
           if(num !=4)
             fvwm_msg(ERR,"ProcessNewStyle",
                      "IconBox style requires 4 arguments!");
-          else
-          {
-            /* check for negative locations */
-            if(IconBox[0] < 0)
-              IconBox[0] += Scr.MyDisplayWidth;
-            if(IconBox[1] < 0)
-              IconBox[1] += Scr.MyDisplayHeight;
-            if(IconBox[2] < 0)
-              IconBox[2] += Scr.MyDisplayWidth;
-            if(IconBox[3] < 0)
-              IconBox[3] += Scr.MyDisplayHeight;
-          }
         }
         else if(mystrncasecmp(restofline,"ICON",4)==0)
         {
@@ -460,7 +457,9 @@ void ProcessNewStyle(XEvent *eventp,
             restofline++;
           while(isspace(*restofline))restofline++;
 	  
-          off_buttons |= (1<<(butt-1));
+          if (butt == 0) butt = 10;
+          if (butt > 0 && butt <= 10)
+            off_buttons |= (1<<(butt-1));
         }
         else if(mystrncasecmp(restofline,"NOOLDECOR",9)==0)
         {
