@@ -80,9 +80,6 @@ void HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
   XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
 		&x, &y, &JunkX, &JunkY, &JunkMask);   
   
-  /* Turn off the rubberband if its on */
-  MoveOutline(Scr.Root,0,0,0,0);
-  
   /* Move the viewport */
   /* and/or move the cursor back to the approximate correct location */
   /* that is, the same place on the virtual desktop that it */
@@ -172,6 +169,8 @@ void HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
     {
       if(Grab)
 	MyXGrabServer(dpy);
+      /* Turn off the rubberband if its on */
+      MoveOutline(Scr.Root,0,0,0,0);
       XWarpPointer(dpy,None,Scr.Root,0,0,0,0,*xl,*yt);
       MoveViewport(Scr.Vx + *delta_x,Scr.Vy + *delta_y,False);
       XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
@@ -532,7 +531,7 @@ void  changeDesks(int val1, int val2)
 void changeWindowsDesk(XEvent *eventp,Window w,FvwmWindow *t,
 		       unsigned long context,char *action, int *Module)
 {
-  long val1, val2;
+  int val1, val2;
   int val1_unit,val2_unit,n;
   
   if (DeferExecution(eventp,&w,&t,&context,SELECT,ButtonRelease))
