@@ -40,6 +40,9 @@
 #ifdef XPM
 #include <X11/xpm.h>
 #endif
+#ifdef SHAPE
+#include <X11/extensions/shape.h>
+#endif
 
 #include "../../configure.h"
 #include "../../fvwm/module.h"
@@ -230,12 +233,11 @@ void DeadPipe(int whatever)
     {
       if(b->flags&b_Icon)
 	DestroyPicture(Dpy,b->icon);
-/*
       if(b->flags&b_IconBack)
 	DestroyPicture(Dpy,b->backicon);
-      if(b->flags&b_Container && b->c->flags&b_IconBack)
+      if(b->flags&b_Container && b->c->flags&b_IconBack
+	 && !(b->c->flags&b_TransBack))
 	DestroyPicture(Dpy,b->c->backicon);
-*/
     }
 
   exit(0);
@@ -323,6 +325,80 @@ char *GetButtonAction(button_info *b,int n)
   return b->action[n];
 }
 
+#ifdef SHAPE
+/**
+*** SetTransparentBackground()
+*** use the Shape extension to create a transparent background.
+***   Patrice Fortier
+**/
+void SetTransparentBackground(button_info *ub,int w,int h)
+{
+  Pixmap pmap_mask;
+  button_info *b;
+  GC gc;
+  XGCValues gvals;
+  unsigned long gcm=0;
+  Window root_return;
+  int x_return, y_return;
+  unsigned int width_return, height_return;
+  unsigned int border_width_return;
+  unsigned int depth_return;
+  int number, i;
+  XFontStruct *font;
+  
+  pmap_mask = XCreatePixmap(Dpy,MyWindow,w,h,1);
+  gc = XCreateGC(Dpy,pmap_mask,(unsigned long)0,&gvals);
+  XSetForeground(Dpy,gc,0);
+  XFillRectangle(Dpy,pmap_mask,gc,0,0,w,h);
+  XSetForeground(Dpy,gc,1);
+  
+  /*
+   * if button has an icon, draw a rect with the same size as the icon
+   * (or the mask of the icon),
+   * else draw a rect with the same size as the button.
+   */
+  
+  i=-1;
+  while(NextButton(&ub,&b,&i,0))
+    {
+      if(b->flags&b_Icon) 
+	{
+	  XGetGeometry(Dpy,b->IconWin,&root_return,&x_return,&y_return,
+		       &width_return,&height_return,
+		       &border_width_return,&depth_return);
+	  
+	  number=buttonNum(b);
+	  if (b->icon->mask == None)
+	    XFillRectangle(Dpy,pmap_mask,gc,x_return, y_return,
+			   b->icon->width,b->icon->height);
+	  else
+	    XCopyArea(Dpy,b->icon->mask,pmap_mask,gc,0,0,
+		      b->icon->width,b->icon->height,x_return,y_return);
+	}
+      else
+	{
+	  number=buttonNum(b);
+	  XFillRectangle(Dpy,pmap_mask,gc,buttonXPos(b,number),
+			 buttonYPos(b,number),
+			 buttonWidth(b),buttonHeight(b));
+	}
+      
+      /* handle button's title */
+      font=buttonFont(b);
+      if(b->flags&b_Title && font)
+	{
+	  gcm = GCForeground | GCFont; 
+	  gvals.foreground=1;
+	  gvals.font = font->fid;
+	  XChangeGC(Dpy,gc,gcm,&gvals);
+	  DrawTitle(b,pmap_mask,gc);
+	}
+    }
+  XFreeGC(Dpy,gc);
+  XShapeCombineMask(Dpy,MyWindow,ShapeBounding,0,0,pmap_mask,ShapeSet);
+}
+#endif
+
 /**
 *** myErrorHandler()
 *** Shows X errors made by FvwmButtons.
@@ -467,6 +543,11 @@ void main(int argc, char **argv)
   while(NextButton(&ub,&b,&i,0))
     ConfigureIconWindow(b);
 
+# ifdef SHAPE
+  if(UberButton->c->flags&b_TransBack)
+    SetTransparentBackground(UberButton,Width,Height);
+# endif
+
 # ifdef DEBUG_INIT
   fprintf(stderr,"OK\n%s: Mapping windows...",MyName);
 # endif
@@ -815,12 +896,12 @@ void RecursiveLoadData(button_info *b,int *maxx,int *maxy)
 	b->c->fc=GetColor(b->c->fore);
       if(b->c->flags&b_Back)
 	{
-	  if(b->c->flags&b_IconBack)
+	  if(b->c->flags&b_IconBack && !(b->c->flags&b_TransBack))
 	    {
-	      if(!LoadIconFile(b->c->back,&b->c->backicon))
-		b->c->flags&=~b_Back;
+	      if(!LoadIconFile(b->c->back_file,&b->c->backicon))
+		b->c->flags&=~b_IconBack;
 	    }
-	  else
+
 	    {
 	      b->c->bc=GetColor(b->c->back);
 	      b->c->hc=GetHilite(b->c->bc);
@@ -1070,6 +1151,10 @@ void CreateWindow(button_info *ub,int maxx,int maxy)
   MyWindow = XCreateSimpleWindow(Dpy,Root,mysizehints.x,mysizehints.y,
 				 mysizehints.width,mysizehints.height,
 				 1,fore_pix,back_pix);
+  if(ub->c->flags&b_IconBack && !(ub->c->flags&b_TransBack))
+    XSetWindowBackgroundPixmap(Dpy,MyWindow,ub->c->backicon->picture);
+
+
 # ifdef DEBUG_INIT
   fprintf(stderr,"properties...");
 # endif
