@@ -29,13 +29,16 @@ extern Display *dpy;
 Pixel back_pix, fore_pix, hi_pix;
 Pixel focus_pix;
 Pixel focus_fore_pix;
+extern Pixel win_back_pix, win_fore_pix, win_hi_back_pix, win_hi_fore_pix;
 extern int window_w, window_h,window_x,window_y,usposition,uselabel,xneg,yneg;
 extern int StartIconic;
+extern int MiniIcons;
 extern int icon_w, icon_h, icon_x, icon_y;
 XFontStruct *font, *windowFont;
 
 GC NormalGC,DashedGC,HiliteGC,rvGC;
 GC StdGC;
+GC MiniIconGC;
 
 extern PagerWindow *Start;
 extern PagerWindow *FocusWin;
@@ -103,6 +106,7 @@ void initialize_pager(void)
   unsigned long valuemask;
   XSetWindowAttributes attributes;
   extern char *PagerFore, *PagerBack, *HilightC;
+  extern char *WindowBack, *WindowFore, *WindowHiBack, *WindowHiFore;
   extern char *font_string, *smallFont;
   int n,m,w,h,i,x,y;
   XGCValues gcv;
@@ -142,6 +146,13 @@ void initialize_pager(void)
   back_pix = GetColor(PagerBack);
   hi_pix = GetColor(HilightC);
 
+  if (WindowBack && WindowFore && WindowHiBack && WindowHiFore)
+  {
+  	win_back_pix	= GetColor (WindowBack);
+  	win_fore_pix	= GetColor (WindowFore);
+  	win_hi_back_pix	= GetColor (WindowHiBack);
+  	win_hi_fore_pix	= GetColor (WindowHiFore);
+  }
   /* Load pixmaps for mono use */
   if(Scr.d_depth<2)
     {
@@ -361,6 +372,7 @@ void initialize_pager(void)
 
   gcv.font =  font->fid;
   NormalGC = XCreateGC(dpy, Scr.Root, gcm, &gcv);  
+  MiniIconGC = XCreateGC(dpy, Scr.Root, gcm, &gcv);  
 
   gcv.foreground = hi_pix;
   if(Scr.d_depth < 2)
@@ -559,10 +571,12 @@ void HandleExpose(XEvent *Event)
       if(t->PagerView == Event->xany.window)
 	{
 	  LabelWindow(t);
+	  PictureWindow(t);
 	}
       else if(t->IconView == Event->xany.window)
 	{
 	  LabelIconWindow(t);
+	  PictureIconWindow(t);
 	}
       
       t = t->next;
@@ -724,6 +738,8 @@ void ReConfigureIcons(void)
       if (h < 1)
 	h = 1;
       
+      t->icon_view_width      = w;
+      t->icon_view_height     = h;
       if(Scr.CurrentDesk == t->desk)
 	XMoveResizeWindow(dpy,t->IconView,x,y,w,h);
       else
@@ -930,6 +946,8 @@ void AddNewWindow(PagerWindow *t)
   if(h<1)
     h = 1;
 
+	t->pager_view_width		= w;
+	t->pager_view_height	= h;
   valuemask = (CWBackPixel | CWBorderPixel | CWEventMask);
   attributes.background_pixel = t->back;
   attributes.border_pixel = fore_pix;
@@ -959,6 +977,8 @@ void AddNewWindow(PagerWindow *t)
   if(h<1)
     h = 1;
 
+	t->icon_view_width	= w;
+	t->icon_view_height	= h;
   if(Scr.CurrentDesk == t->desk)
     {
       t->IconView = XCreateWindow(dpy,icon_win, x, y, w, h,1,
@@ -1010,6 +1030,8 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
     w = 1;
   if (h < 1)
     h = 1;
+	t->pager_view_width		= w;
+	t->pager_view_height	= h;
   if((i >= 0)&&(i < ndesks))
     {
       XReparentWindow(dpy, t->PagerView, Desks[i].w, x,y);
@@ -1030,6 +1052,8 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
     w = 1;
   if (h < 1)
     h = 1;
+	t->icon_view_width	= w;
+	t->icon_view_height	= h;
   if(Scr.CurrentDesk == t->desk)
     XMoveResizeWindow(dpy,t->IconView,x,y,w,h);
   else
@@ -1055,6 +1079,8 @@ void MoveResizePagerView(PagerWindow *t)
     w = 1;
   if (h < 1)
     h = 1;
+	t->pager_view_width		= w;
+	t->pager_view_height	= h;
   if(t->PagerView != None)
     XMoveResizeWindow(dpy,t->PagerView,x,y,w,h);
   else if((t->desk >= desk1)&&(t->desk <= desk2))
@@ -1075,6 +1101,8 @@ void MoveResizePagerView(PagerWindow *t)
     w = 1;
   if (h < 1)
     h = 1;
+	t->icon_view_width	= w;
+	t->icon_view_height	= h;
   if(Scr.CurrentDesk == t->desk)
     XMoveResizeWindow(dpy,t->IconView,x,y,w,h);
   else
@@ -1159,6 +1187,8 @@ void Hilight(PagerWindow *t, int on)
   XClearWindow(dpy,t->IconView);
   LabelWindow(t);
   LabelIconWindow(t);
+  PictureWindow(t);
+  PictureIconWindow(t);
 }
 
 void Scroll(int Desk, int x, int y)
@@ -1492,6 +1522,10 @@ void LabelWindow(PagerWindow *t)
     {
       return;
     }
+  if (MiniIcons && t->mini_icon.picture && (t->PagerView != None))
+  {
+    return; /* will draw picture instead... */
+  }
   if(t->icon_name == NULL)
     {
       return;
@@ -1529,6 +1563,10 @@ void LabelIconWindow(PagerWindow *t)
     {
       return;
     }
+  if (MiniIcons && t->mini_icon.picture && (t->PagerView != None))
+  {
+    return; /* will draw picture instead... */
+  }
   if(t->icon_name == NULL)
     {
       return;
@@ -1539,6 +1577,7 @@ void LabelIconWindow(PagerWindow *t)
       Globalgcv.foreground = focus_fore_pix;
       Globalgcv.background = focus_pix;
       Globalgcm = GCForeground|GCBackground;
+      XChangeGC(dpy,StdGC,Globalgcm,&Globalgcv); 
     }
   else
     {
@@ -1553,6 +1592,92 @@ void LabelIconWindow(PagerWindow *t)
 	       t->icon_name, strlen(t->icon_name));        
 
 }
+void PictureWindow (PagerWindow *t)
+{
+	XGCValues		Globalgcv;
+	unsigned long	Globalgcm;
+	int				iconX;
+	int				iconY;
+	if (MiniIcons)
+	{
+		if (t->mini_icon.picture && (t->PagerView != None))
+		{
+			if (t->pager_view_width > t->mini_icon.width)
+				iconX = (t->pager_view_width - t->mini_icon.width) / 2;
+			else if (t->pager_view_width < t->mini_icon.width)
+				iconX = -((t->mini_icon.width - t->pager_view_width) / 2);
+			else
+				iconX = 0;
+			if (t->pager_view_height > t->mini_icon.height)
+				iconY = (t->pager_view_height - t->mini_icon.height) / 2;
+			else if (t->pager_view_height < t->mini_icon.height)
+				iconY = -((t->mini_icon.height - t->pager_view_height) / 2);
+			else
+				iconY = 0;
+			Globalgcm				= GCForeground | GCBackground | GCClipMask | GCClipXOrigin | GCClipYOrigin;
+			Globalgcv.clip_mask		= t->mini_icon.mask;
+			Globalgcv.clip_x_origin	= iconX;
+			Globalgcv.clip_y_origin	= iconY;
+			if (t == FocusWin)
+			{
+				Globalgcv.foreground	= focus_fore_pix;
+				Globalgcv.background	= focus_pix;
+			}
+			else
+			{
+				Globalgcv.foreground	= t->text;
+				Globalgcv.background	= t->back;
+			}
+			XChangeGC (dpy, MiniIconGC, Globalgcm, &Globalgcv);
+			XClearWindow (dpy, t->PagerView);
+			XCopyArea (dpy, t->mini_icon.picture, t->PagerView, MiniIconGC,
+							0, 0, t->mini_icon.width, t->mini_icon.height, iconX, iconY);
+		}
+	}
+}
+void PictureIconWindow (PagerWindow *t)
+{
+	XGCValues		Globalgcv;
+	unsigned long	Globalgcm;
+	int				iconX;
+	int				iconY;
+	if (MiniIcons)
+	{
+		if (t->mini_icon.picture && (t->IconView != None))
+		{
+			if (t->icon_view_width > t->mini_icon.width)
+				iconX = (t->icon_view_width - t->mini_icon.width) / 2;
+			else if (t->icon_view_width < t->mini_icon.width)
+				iconX = -((t->mini_icon.width - t->icon_view_width) / 2);
+			else
+				iconX = 0;
+			if (t->icon_view_height > t->mini_icon.height)
+				iconY = (t->icon_view_height - t->mini_icon.height) / 2;
+			else if (t->icon_view_height < t->mini_icon.height)
+				iconY = -((t->mini_icon.height - t->icon_view_height) / 2);
+			else
+				iconY = 0;
+			Globalgcm				= GCForeground | GCBackground | GCClipMask | GCClipXOrigin | GCClipYOrigin;
+			Globalgcv.clip_mask		= t->mini_icon.mask;
+			Globalgcv.clip_x_origin	= iconX;
+			Globalgcv.clip_y_origin	= iconY;
+			if (t == FocusWin)
+			{
+				Globalgcv.foreground	= focus_fore_pix;
+				Globalgcv.background	= focus_pix;
+			}
+			else
+			{
+				Globalgcv.foreground	= t->text;
+				Globalgcv.background	= t->back;
+			}
+			XChangeGC (dpy, MiniIconGC, Globalgcm, &Globalgcv);
+			XClearWindow (dpy, t->IconView);
+			XCopyArea (dpy, t->mini_icon.picture, t->IconView, MiniIconGC,
+							0, 0, t->mini_icon.width, t->mini_icon.height, iconX, iconY);
+		}
+	}
+}
 
 void IconMoveWindow(XEvent *Event,PagerWindow *t)
 {
