@@ -16,12 +16,29 @@
  * No guarantees or warantees or anything
  * are provided or implied in any way whatsoever. Use this program at your
  * own risk. Permission to use this program for any purpose is given,
- * as long as the copyright is kept intact. */
+ * as long as the copyright is kept intact.
+
+ * Modifications Done to Add Pixmaps, focus highlighting and Current Desk Only
+ * By Don Mahurin, 1996, Some of this Code was taken from FvwmTaskBar
+
+ * Bug Notes:(Don Mahurin)
+
+ * Moving a window doesnt send M_CONFIGURE, as I thought it should. Desktop
+ * for that button is not updated.
+
+ * There seems to be something incosistant about Resizing and the value given
+ * back as win_x, win_y.  SEE RESIZE_BUG_HACK for my work around. 
+ 
+ * Don Mahurin (dmahurin@donet.com)
+
+*/
 
 #define TRUE 1
 #define FALSE 
 
+#ifndef NO_CONSOLE
 #define NO_CONSOLE
+#endif
 
 #ifndef min
 #define min(a,b) (((a)<(b)) ? (a) : (b))
@@ -58,6 +75,7 @@
 
 #include "../../fvwm/module.h"
 #include "../../version.h"
+
 #include "FvwmWinList.h"
 #include "ButtonArray.h"
 #include "List.h"
@@ -100,6 +118,20 @@ char *ClickAction[3]={"Iconify -1,Raise","Iconify","Lower"},*EnterAction,
 char *font_string = "fixed";
 int UseSkipList=0,Anchor=1,UseIconNames=0,LeftJustify=0,TruncateLeft=1;
 
+long CurrentDesk = 0;
+int ShowCurrentDesk = 0;
+
+int RESIZE_BUG_HACK = 0;
+
+int ItemCountD(List *list )
+{
+	if(!ShowCurrentDesk)
+		return ItemCount(list);
+	/* else */
+	return ItemCountDesk(list, CurrentDesk);
+
+}
+
 /******************************************************************************
   Main - Setup the XConnection,request the window list and loop forever
     Based on main() from FvwmIdent:
@@ -107,10 +139,7 @@ int UseSkipList=0,Anchor=1,UseIconNames=0,LeftJustify=0,TruncateLeft=1;
 ******************************************************************************/
 void main(int argc, char **argv)
 {
-char *temp, *s;
-
-  /* Open the console for messages */
-  OpenConsole();
+  char *temp, *s;
 
   /* Save the program name for error messages and config parsing */
   temp = argv[0];
@@ -124,6 +153,9 @@ char *temp, *s;
   strcat(Module, temp);
   Clength = strlen(Module);
 
+  /* Open the console for messages */
+  OpenConsole();
+
   if((argc != 6)&&(argc != 7)) {
     fprintf(stderr,"%s Version %s should only be executed by fvwm!\n",Module,
       VERSION);
@@ -145,19 +177,32 @@ char *temp, *s;
 
   /* Setup the XConnection */
   StartMeUp();
+  XSetErrorHandler((XErrorHandler) ErrorHandler);
 
-  InitArray(&buttons,0,0,win_width,fontheight+6);
+  InitPictureCMap(dpy, Root);
+
+  InitArray(&buttons,0,0,win_width, fontheight+6);
   InitList(&windows);
 
   fd_width = GetFdWidth();
 
   /* Request a list of all windows,
    * wait for ConfigureWindow packets */
-  SendFvwmPipe("Send_WindowList",0);
 
 /*  SetMessageMask(Fvwm_fd,M_ADD_WINDOW|M_CONFIGURE_WINDOW| M_DESTROY_WINDOW|
 	      M_WINDOW_NAME|M_ICON_NAME|M_DEICONIFY|M_ICONIFY|M_END_WINDOWLIST|
 	      M_NEW_DESK| M_NEW_PAGE);*/
+  SetMessageMask(Fvwm_fd,M_CONFIGURE_WINDOW | M_RES_CLASS | M_RES_NAME |
+                 M_ADD_WINDOW | M_DESTROY_WINDOW | M_ICON_NAME |
+                 M_DEICONIFY | M_ICONIFY | M_END_WINDOWLIST |
+                 M_NEW_DESK | M_NEW_PAGE | M_FOCUS_CHANGE | M_WINDOW_NAME |
+#ifdef MINI_ICONS
+                 M_MINI_ICON |
+#endif
+                 M_STRING);
+
+  SendFvwmPipe("Send_WindowList",0);
+
   /* Recieve all messages from Fvwm */
   EndLessLoop();
 }
@@ -223,31 +268,69 @@ void ReadFvwmPipe()
 ******************************************************************************/
 void ProcessMessage(unsigned long type,unsigned long *body)
 {
-int redraw=0,i;
-long flags;
-char *name,*string;
-static current_focus=-1;
+  int redraw=0,i;
+  long flags;
+  char *name,*string;
+  static current_focus=-1;
 
-  switch(type) {
+  Picture p;
+
+  switch(type)
+  {
     case M_ADD_WINDOW:
     case M_CONFIGURE_WINDOW:
-      if (body[0]==win) {
+      if (body[0] == win)
+      {
         win_x=(int)body[3];
         win_y=(int)body[4];
+
         win_title=(int)body[9];
         win_border=(int)body[10];
-        if (win_border!=0) win_border++;
       }
-      if (FindItem(&windows,body[0])!=-1) break;
+      if ((i = FindItem(&windows,body[0]))!=-1) 
+      { 
+/*	printf("does it go here on MOVE %ld, %ld ?\n", i, CurrentDesk);
+*/
+	if(UpdateItemDesk(&windows, i, body[7]) > 0)
+        {
+/*         printf("why doesnt it go here on MOVE ?\n");
+*/
+          AdjustWindow();
+          RedrawWindow(1);
+        }
+	break;
+      }
+
       if (!(body[8]&WINDOWLISTSKIP) || !UseSkipList)
-        AddItem(&windows,body[0],body[8]);
+        AddItem(&windows,body[0],body[8], body[7] /* desk */);
       break;
     case M_DESTROY_WINDOW:
       if ((i=DeleteItem(&windows,body[0]))==-1) break;
       RemoveButton(&buttons,i);
-      if (WindowIsUp) AdjustWindow();
+      if (WindowIsUp)
+        AdjustWindow();
+
       redraw=1;
       break;
+
+#ifdef MINI_ICONS
+  case M_MINI_ICON:
+    if ((i=FindItem(&windows,body[0]))==-1) break;
+
+    if (UpdateButton(&buttons,i,NULL,-1)!=-1)
+    {
+      p.width = body[3];
+      p.height = body[4];
+      p.depth = body[5];
+      p.picture = body[6];
+      p.mask = body[7];
+
+      UpdateButtonPicture(&buttons, i, &p);
+      redraw = 0;
+    }
+    break;
+#endif
+
     case M_WINDOW_NAME:
     case M_ICON_NAME:
       if ((type==M_ICON_NAME && !UseIconNames) || 
@@ -257,8 +340,9 @@ static current_focus=-1;
       name=makename(string,ItemFlags(&windows,body[0]));
       if (UpdateButton(&buttons,i,name,-1)==-1)
       {
-        AddButton(&buttons,name,1);
+	AddButton(&buttons, name, NULL, 1);
 	UpdateButtonSet(&buttons,i,ItemFlags(&windows,body[0])&ICONIFIED?1:0);
+        UpdateButtonDesk(&buttons,i,ItemDesk(&windows, body[0]));
       }
       free(name);
       if (WindowIsUp) AdjustWindow();
@@ -280,6 +364,9 @@ static current_focus=-1;
       free(name);
       break;
     case M_FOCUS_CHANGE:
+
+/* old coded, before Task bar code spliced in
+
       i=FindItem(&windows,body[0]);
       if (i==current_focus) break;
       if (current_focus!=-1)
@@ -290,12 +377,33 @@ static current_focus=-1;
       current_focus=i;
       if (current_focus!=-1)
 	if (UpdateButtonSet(&buttons,current_focus,2)!=-1) redraw=1;
+* end of old code */
+
+/* Code copied Straight from TaskBar */
+
+    if ((i=FindItem(&windows,body[0]))!=-1)
+    {
+	    flags=ItemFlags(&windows,body[0]);
+ 	    UpdateItemFlags(&windows,body[0],flags);
+    }
+    RadioButton(&buttons,i);
+    redraw = 1;
+    break;
+
       break;
     case M_END_WINDOWLIST:
       if (!WindowIsUp) MakeMeWindow();
       redraw=1;
       break;
     case M_NEW_DESK:
+	CurrentDesk = body[0];
+	if(ShowCurrentDesk)
+	{
+	  RESIZE_BUG_HACK = 1;
+	  AdjustWindow();
+	  RESIZE_BUG_HACK = 0;
+	  RedrawWindow(1);
+        }
       break;
     case M_NEW_PAGE:
       break;
@@ -311,13 +419,16 @@ static current_focus=-1;
 ******************************************************************************/
 void SendFvwmPipe(char *message,unsigned long window)
 {
-int w;
-char *hold,*temp,*temp_msg;
+  int w;
+  char *hold,*temp,*temp_msg;
+
   hold=message;
 
-  while(1) {
+  while(1)
+  {
     temp=strchr(hold,',');
-    if (temp!=NULL) {
+    if (temp!=NULL)
+    {
       temp_msg=malloc(temp-hold+1);
       strncpy(temp_msg,hold,(temp-hold));
       temp_msg[(temp-hold)]='\0';
@@ -354,10 +465,13 @@ void DeadPipe(int nonsense)
 ******************************************************************************/
 void WaitForExpose(void)
 {
-XEvent Event;
-  while(1) {
+  XEvent Event;
+
+  while(1)
+  {
     XNextEvent(dpy,&Event);
-    if (Event.type==Expose) {
+    if (Event.type==Expose)
+    {
       if (Event.xexpose.count==0) break;
     }
   }
@@ -368,7 +482,7 @@ XEvent Event;
 ******************************************************************************/
 void RedrawWindow(int force)
 {
-  DrawButtonArray(&buttons,force);
+  DrawButtonArray(&buttons, force);
   if (XQLength(dpy) && !force) LoopOnEvents();
 }
 
@@ -378,8 +492,9 @@ void RedrawWindow(int force)
 void ConsoleMessage(char *fmt, ...)
 {
 #ifndef NO_CONSOLE
-va_list args;
-FILE *filep;
+  va_list args;
+  FILE *filep;
+
   if (console==NULL) filep=stderr;
   else filep=console;
   va_start(args,fmt);
@@ -440,6 +555,8 @@ void ParseConfig()
 				Clength+11)==0) UseSkipList=1;
 	  else if(mystrncasecmp(tline,CatString3(Module, "UseIconNames",""),
 				Clength+12)==0) UseIconNames=1;
+	  else if(mystrncasecmp(tline,CatString3(Module, "ShowCurrentDesk",""),
+				Clength+15)==0) ShowCurrentDesk=1;
 	  else if(mystrncasecmp(tline,CatString3(Module, "LeftJustify",""),
 				Clength+11)==0) LeftJustify=1;
 	  else if(mystrncasecmp(tline,CatString3(Module, "TruncateLeft",""),
@@ -460,17 +577,19 @@ void ParseConfig()
 ******************************************************************************/
 void LoopOnEvents()
 {
-int num;
-char buffer[10];
-XEvent Event;
-Window dummyroot,dummychild;
-int x,x1,y,y1;
-unsigned int dummy1;
+  int num;
+  char buffer[10];
+  XEvent Event;
+  Window dummyroot,dummychild;
+  int x,x1,y,y1;
+  unsigned int dummy1;
 
-  if (Transient && !Checked) {
+  if (Transient && !Checked)
+  {
     XQueryPointer(dpy,win,&dummyroot,&dummychild,&x1,&y1,&x,&y,&dummy1);
     num=WhichButton(&buttons,x,y);
-    if (num!=-1) {
+    if (num!=-1)
+    {
       Pressed=1;
       ButPressed=num;
       SwitchButton(&buttons,num);
@@ -478,14 +597,18 @@ unsigned int dummy1;
     Checked=1;
   }
 
-  while(XPending(dpy)) {
+  while(XPending(dpy))
+  {
     XNextEvent(dpy,&Event);
 
-    switch(Event.type) {
+    switch(Event.type)
+    {
       case ButtonRelease:
-        if (Pressed) {
+        if (Pressed)
+        {
           num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
-          if (num!=-1) {
+          if (num!=-1)
+          {
             SendFvwmPipe(ClickAction[(Transient) ? 0:Event.xbutton.button-1],
               ItemID(&windows,num));
             SwitchButton(&buttons,num);
@@ -497,7 +620,8 @@ unsigned int dummy1;
         break;
       case ButtonPress:
         num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
-        if (num != -1) {
+        if (num != -1)
+        {
           SwitchButton(&buttons,num);
           ButPressed=num;
         } else ButPressed=-1;
@@ -509,7 +633,8 @@ unsigned int dummy1;
         break;
       case KeyPress:
         num=XLookupString(&Event.xkey,buffer,10,NULL,0);
-        if (num==1) {
+        if (num==1)
+        {
           if (buffer[0]=='q' || buffer[0]=='Q') ShutMeDown(0);
           else if (buffer[0]=='i' || buffer[0]=='I') PrintList(&windows);
           else if (buffer[0]=='b' || buffer[0]=='B') PrintButtons(&buttons);
@@ -521,7 +646,8 @@ unsigned int dummy1;
       case EnterNotify:
         if (!SomeButtonDown(Event.xcrossing.state)) break;
         num=WhichButton(&buttons,Event.xcrossing.x,Event.xcrossing.y);
-        if (num!=-1) {
+        if (num!=-1)
+        {
           SwitchButton(&buttons,num);
           ButPressed=num;
         } else ButPressed=-1;
@@ -537,10 +663,13 @@ unsigned int dummy1;
         num=WhichButton(&buttons,Event.xmotion.x,Event.xmotion.y);
         if (num==ButPressed) break;
         if (ButPressed!=-1) SwitchButton(&buttons,ButPressed);
-        if (num!=-1) {
+        if (num!=-1)
+        {
           SwitchButton(&buttons,num);
           ButPressed=num;
-        } else ButPressed=-1;
+        }
+        else ButPressed=-1;
+
         break;
     }
   }
@@ -551,44 +680,72 @@ unsigned int dummy1;
 ******************************************************************************/
 void AdjustWindow()
 {
-int new_width=0,new_height=0,tw,i,total;
-char *temp;
+  int new_width=0,new_height=0,tw,i,total;
+  char *temp;
 
-  total=ItemCount(&windows);
-  if (!total) {
-    if (WindowIsUp==1) {
+  total = ItemCountD(&windows );
+  if (!total)
+  {
+    if (WindowIsUp==1)
+    {
       XUnmapWindow(dpy,win);
       WindowIsUp=2;
     }
     return;
   }
-  for(i=0;i<total;i++) {
+  for(i=0;i<total;i++)
+  {
     temp=ItemName(&windows,i);
     if(temp != NULL)
-      {
+    {
 	tw=10+XTextWidth(ButtonFont,temp,strlen(temp));
 	tw+=XTextWidth(ButtonFont,"()",2);
+
+	tw+=14; /* for title icon */ /* Magic Number ? */
+
 	new_width=max(new_width,tw);
-      }
+    }
   }
-  new_width=max(new_width,MinWidth);
-  new_width=min(new_width,MaxWidth);
+  new_width=max(new_width, MinWidth);
+  new_width=min(new_width, MaxWidth);
   new_height=(total*(fontheight+6+1)-1);
-  if (WindowIsUp && (new_height!=win_height  || new_width!=win_width)) {
-    if (Anchor) {
+  if (WindowIsUp && (new_height!=win_height  || new_width!=win_width))
+  {
+    if (Anchor)
+    {
       if (win_grav==SouthEastGravity || win_grav==NorthEastGravity)
         win_x-=(new_width-win_width);
+
       if (win_grav==SouthEastGravity || win_grav==SouthWestGravity)
         win_y-=(new_height-win_height);
+
+      if(RESIZE_BUG_HACK)
+      {
+        if (win_grav==SouthEastGravity || win_grav==NorthEastGravity)
+		win_x+= win_border - 2;
+	else
+		win_x-=win_border;
+
+	if (win_grav==SouthEastGravity || win_grav==SouthWestGravity)
+		win_y+=win_border - 2;
+	else	
+		win_y-=win_border;
+      }
+
       XMoveResizeWindow(dpy,win,win_x+win_border,win_y+win_title+win_border,
         new_width,new_height);
-    } else XResizeWindow(dpy,win,new_width,new_height);
+
+    }
+    else
+      XResizeWindow(dpy, win, new_width,new_height);
+
     XSync(dpy,False);
   }
   UpdateArray(&buttons,-1,-1,new_width,-1);
-  if (new_height>0) win_height=new_height;
-  if (new_width>0) win_width=new_width;
-  if (WindowIsUp==2) {
+  if (new_height>0) win_height = new_height;
+  if (new_width>0) win_width = new_width;
+  if (WindowIsUp==2)
+  {
     XMapWindow(dpy,win);
     WindowIsUp=1;
     WaitForExpose();
@@ -632,15 +789,16 @@ char *temp;
 ******************************************************************************/
 void MakeMeWindow(void)
 {
-XSizeHints hints;
-XGCValues gcval;
-unsigned long gcmask;
-unsigned int dummy1,dummy2;
-int x,y,ret,count;
-Window dummyroot,dummychild;
-int i;
+  XSizeHints hints;
+  XGCValues gcval;
+  unsigned long gcmask;
+  unsigned int dummy1, dummy2;
+  int x, y, ret, count;
+  Window dummyroot, dummychild;
+  int i;
+
   
-  if ((count=ItemCount(&windows))==0 && Transient) ShutMeDown(0);
+  if ((count = ItemCountD(&windows))==0 && Transient) ShutMeDown(0);
   AdjustWindow();
 
   hints.width=win_width;
@@ -650,28 +808,38 @@ int i;
   hints.width_inc=0;
   hints.height_inc=0;
 
-  if (geometry!= NULL) {
+  if (geometry!= NULL)
+  {
     ret=XParseGeometry(geometry,&x,&y,&dummy1,&dummy2);
 
-    if (ret&XValue && ret &YValue) {
+    if (ret&XValue && ret &YValue)
+    {
       hints.x=x;
-      if (ret&XNegative) hints.x+=XDisplayWidth(dpy,screen)-win_width;
+      if (ret&XNegative)
+        hints.x+=XDisplayWidth(dpy,screen)-win_width;
+
       hints.y=y;
-      if (ret&YNegative) hints.y+=XDisplayHeight(dpy,screen)-win_height;
+      if (ret&YNegative)
+        hints.y+=XDisplayHeight(dpy,screen)-win_height;
+
       hints.flags|=USPosition;
     }
 
-    if (ret&XNegative) {
+    if (ret&XNegative)
+    {
       if (ret&YNegative) hints.win_gravity=SouthEastGravity;
       else  hints.win_gravity=NorthEastGravity;
-    } else {
+    }
+    else
+    {
       if (ret&YNegative) hints.win_gravity=SouthWestGravity;
       else  hints.win_gravity=NorthWestGravity;
     }
 
   }
 
-  if (Transient) {
+  if (Transient)
+  {
     XQueryPointer(dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,&dummy1);
     hints.win_gravity=NorthWestGravity;
     hints.flags |= USPosition;
@@ -680,11 +848,15 @@ int i;
   win_x=hints.x;
   win_y=hints.y;
 
+
   for (i = 0; i != MAX_COLOUR_SETS; i++)
-  if(d_depth < 2) {
+  if(d_depth < 2)
+  {
     back[i] = GetColor("white");
     fore[i] = GetColor("black");
-  } else {
+  }
+  else
+  {
     back[i] = GetColor(BackColor[i] == NULL ? BackColor[0] : BackColor[i]);
     fore[i] = GetColor(ForeColor[i] == NULL ? ForeColor[0] : ForeColor[i]);
   }
@@ -697,7 +869,8 @@ int i;
   
   XSetWMNormalHints(dpy,win,&hints);
 
-  if (!Transient) {
+  if (!Transient)
+  {
     XGrabButton(dpy,1,AnyModifier,win,True,GRAB_EVENTS,GrabModeAsync,
       GrabModeAsync,None,None);
     XGrabButton(dpy,2,AnyModifier,win,True,GRAB_EVENTS,GrabModeAsync,
@@ -707,7 +880,9 @@ int i;
     SetMwmHints(MWM_DECOR_ALL|MWM_DECOR_RESIZEH|MWM_DECOR_MAXIMIZE|MWM_DECOR_MINIMIZE,
       MWM_FUNC_ALL|MWM_FUNC_RESIZE|MWM_FUNC_MAXIMIZE|MWM_FUNC_MINIMIZE,
       MWM_INPUT_MODELESS);
-  } else {
+  }
+  else
+  {
     SetMwmHints(0,MWM_FUNC_ALL,MWM_INPUT_MODELESS);
   }
 
@@ -741,13 +916,15 @@ int i;
 
   ChangeWindowName(&Module[1]);
 
-  if (ItemCount(&windows)>0) {
+  if (ItemCountD(&windows) > 0)
+  {
     XMapRaised(dpy,win);
     WaitForExpose();
     WindowIsUp=1;
   } else WindowIsUp=2;
 
-  if (Transient) {
+  if (Transient)
+  {
     if ( XGrabPointer(dpy,win,True,GRAB_EVENTS,GrabModeAsync,GrabModeAsync,
       None,None,CurrentTime)!=GrabSuccess) ShutMeDown(1);
     XQueryPointer(dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,&dummy1);
@@ -761,7 +938,8 @@ int i;
 ******************************************************************************/
 void StartMeUp()
 {
-  if (!(dpy = XOpenDisplay(""))) {
+  if (!(dpy = XOpenDisplay("")))
+  {
     fprintf(stderr,"%s: can't open display %s", Module,
       XDisplayName(""));
     exit (1);
@@ -774,14 +952,14 @@ void StartMeUp()
   ScreenHeight = DisplayHeight(dpy,screen);
   ScreenWidth = DisplayWidth(dpy,screen);
 
-  if ((ButtonFont=XLoadQueryFont(dpy,font_string))==NULL) {
+  if ((ButtonFont=XLoadQueryFont(dpy,font_string))==NULL)
+  {
     if ((ButtonFont=XLoadQueryFont(dpy,"fixed"))==NULL) exit(1);
   }
 
-  fontheight=ButtonFont->ascent+ButtonFont->descent;
+  fontheight = ButtonFont->ascent+ButtonFont->descent;
 
   win_width=XTextWidth(ButtonFont,"XXXXXXXXXXXXXXX",10);
- 
 
 }
 
@@ -851,4 +1029,16 @@ PropMwmHints prop;
     }
 }
 
+/************************************************************************
+  X Error Handler
+************************************************************************/
+XErrorHandler ErrorHandler(Display *d, XErrorEvent *event)
+  {
+  char errmsg[256];
+
+  XGetErrorText(d, event->error_code, errmsg, 256);
+  ConsoleMessage("%s failed request: %s\n", Module, errmsg);
+  ConsoleMessage("Major opcode: 0x%x, resource id: 0x%x\n",
+                  event->request_code, event->resourceid);
+  }
 
